import os

from dotenv import load_dotenv
from pathlib import Path  # python3 only
from enum import Enum, unique

# set path to env file
env_path = Path('.') / '.env'
load_dotenv(dotenv_path=env_path)


@unique
class Config(Enum):
    """
    Enum used for storing environment variables
    """
    API_KEY_WEATHER = os.getenv('API_KEY_WEATHER')

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum, unique


@unique
class BasicPhrases(Enum):
    """
    Enum used for storing hard coded phrases, which personal assistant will say
    """
    GOOD_MORNING = "Good Morning"
    GOOD_AFTERNOON = "Good Afternoon"
    GOOD_EVENING = "Good Evening"
    INCOMPREHENSIBLE_SOUND = "Sphinx could not understand audio can you repeat"
    ERROR = "Sphinx error"
    GOODBYE = "GoodBye"
    JOKE = "I have a joke for u"
    END = "See you again"
    START = "Ready Set Go"
    FINISH = "Finish"
    INTRODUCE = "I am Jarvis"
    SAD = "You seem sad"
    NO_COMMEND = "No match for commend"
    HELP = "How may I help you"

    def __str__(self) -> str:
        return str(self.value)


@unique
class JarvisPhrases(Enum):
    """
    Enum used for storing hard coded phrases, which Jarvis will say
    """
    NO_NOTES = "There are no notes saved"
    NO_CITY = "City Not Found"
    BROWSE = "What do you want to browse"
    GOOGLE = "What do you want to google"
    WIKIPEDIA = "What do you want to search in wikipedia",
    WEATHER = "Where do you want to check the weather"

    CREATE_NOTE = "What note"
    HOW_LONG = "How long"

    def __str__(self) -> str:
        return str(self.value)



import datetime
import webbrowser as wb
import requests
import pytemperature
import wikipedia as wiki

from pymongo import DESCENDING, MongoClient


from .personal_assistant import PersonalAssistant
from .config import Config
from .phrases import JarvisPhrases


class Jarvis(PersonalAssistant):
    """
    A class to represent Jarvis an extended version of voice personal assistant.
    Subclass of ``PersonalAssistant``

    ...

    Attributes
    ----------
    mongo_client: pymongo.mongo_client.MongoClient
        Client for a MongoDB instance
    jarvis_database: pymongo.database.Database
        independent MongoDB database created for jarvis
    notes: pymongo.collection.Collection
        independent collection created inside jarvis_database for storing notes
    api_key_weather: str
        key to using weather api from "https://openweathermap.org/api"
    """
    def __init__(self):
        """
        Creates a new ``Jarvis`` instance.
        """
        super().__init__("Jarvis")
        self.mongo_client = MongoClient()
        self.jarvis_database = self.mongo_client.jarvis_database
        self.notes = self.jarvis_database.notes
        self.api_key_weather = Config.API_KEY_WEATHER.value
        self.commands.update({
            "browse": self.browse,
            "search": self.search,
            "stopper": self.stopper,
            "google": self.google,
            "weather": self.weather,
            "take": self.take_note,
            "note": self.read_note,
        })

    def browse(self, source, params):
        """
        Jarvis will open url, created by using first item(word) in params.
        If params are empty Jarvis will aks to provide extra word.

        :param source: speech_recognition.Microphone
        object of speech_recognition.Microphone,  which represents a physical microphone on the computer
        :param params: part of commend after "hot" word
        :return: None
        """
        if not params:
            self.convert_text_to_speech(JarvisPhrases.BROWSE)
            audio = self.recognizer.listen(source)
            browse_for = self.recognizer.recognize_google(audio)
            url = f"www.{browse_for}.com"
        else:
            url = f"www.{params[0]}.com"

        wb.open(url)

    def google(self, source, params):
        """
        Jarvis will google sentence created using joined params.
        If params are empty Jarvis will aks to provide extra words.

        :param source: speech_recognition.Microphone
        object of speech_recognition.Microphone,  which represents a physical microphone on the computer
        :param params: part of commend after "hot" word
        :return: None
        """
        if not params:
            self.convert_text_to_speech("What do you want to google")
            audio = self.recognizer.listen(source, timeout=1)
            google_for = self.recognizer.recognize_google(audio)
            query = google_for.replace(" ", "+")
        else:
            query = "+".join(params)
        url = f"www.google.com/search?q={query}"

        wb.open(url)

    def search(self, source, params):
        """
        Jarvis will search in wikipedia sentence created using joined params.
        If params are empty Jarvis will aks to provide extra words.

        :param source: speech_recognition.Microphone
        object of speech_recognition.Microphone,  which represents a physical microphone on the computer
        :param params: part of commend after "hot" word
        :return: None
        """
        if not params:
            self.convert_text_to_speech(JarvisPhrases.WIKIPEDIA)
            audio = self.recognizer.listen(source)
            search_for = self.recognizer.recognize_google(audio)
            result = wiki.search(search_for)[0]
        else:
            result = wiki.search(" ".join(params))[0]
        page = wiki.page(result)
        url = page.url

        wb.open(url)

    def stopper(self, source, params):
        """
        Jarvis will set a stopper. Time will be calculated depending by params.
        User needs to provide value and unit e.g. 50 seconds.
        If params are empty Jarvis will ask for time.

        :param source: speech_recognition.Microphone
        object of speech_recognition.Microphone,  which represents a physical microphone on the computer
        :param params: part of commend after "hot" word
        :return: None
        """
        if not params:
            self.convert_text_to_speech(JarvisPhrases.HOW_LONG)
            audio = self.recognizer.listen(source)
            text = self.recognizer.recognize_google(audio)
            num, time_unit = text.lower().split()
        else:
            [elem.lower() for elem in params]
            num, time_unit = params

        multipliers = {
            "seconds": 1,
            "minutes": 60,
            "hours": 3600,
        }
        multiplier = [value for (key, value) in multipliers.items()
                      if time_unit == key][0]

        duration = int(num) * multiplier

        self.convert_text_to_speech(JarvisPhrases.START)
        start = datetime.datetime.now()
        end = start + datetime.timedelta(0, duration)
        while datetime.datetime.now() < end:
            pass

        self.convert_text_to_speech(JarvisPhrases.FINISH)

    def weather(self, source, params):
        """
        Jarvis will give you current weather in given city.
        If params are empty Jarvis will ask for name of city.

        :param source: speech_recognition.Microphone
        object of speech_recognition.Microphone,  which represents a physical microphone on the computer
        :param params: part of commend after "hot" word
        :return: None
        """
        if not params:
            self.convert_text_to_speech(JarvisPhrases.WEATHER)
            audio = self.recognizer.listen(source)
            city_name = self.recognizer.recognize_google(audio)
        else:
            city_name = params[0]
        base_url = "http://api.openweathermap.org/data/2.5/weather?"

        complete_url = f"{base_url}appid={self.api_key_weather}&q={city_name}"
        response = requests.get(complete_url).json()
        if response["cod"] != "404":

            weather = response["main"]

            current_temperature = round(pytemperature.k2c(weather["temp"]), 2)
            current_temperature = f'{"minus" if current_temperature < 0 else ""}' + str(current_temperature)
            current_pressure = weather["pressure"]

            current_humidity = weather["humidity"]

            weather_description = response["weather"][0]["description"]

            self.convert_text_to_speech(f'Temperature (in celsius unit) = {current_temperature} '
                                        f'atmospheric pressure (in hPa unit) = {current_pressure} '
                                        f' humidity (in percentage) = {current_humidity} '
                                        f'description = {weather_description}')
        else:
            self.convert_text_to_speech(JarvisPhrases.NO_CITY)

    def take_note(self, source):
        """
        Jarvis will ask for text of note and save it to local database.

        :param source: speech_recognition.Microphone
        object of speech_recognition.Microphone,  which represents a physical microphone on the computer
        :return: None
        """
        self.convert_text_to_speech(JarvisPhrases.CREATE_NOTE)

        audio = self.recognizer.listen(source)
        text = self.recognizer.recognize_google(audio)
        note = {
            "date": datetime.datetime.now().strftime("%c"),
            "text": text
        }
        self.notes.insert_one(note)

    def read_note(self, _):
        """
        Jarvis will read last taken note. If no notes was taken user will be informed.

        :return: None
        """
        try:
            last_note = self.notes.find().sort("date", DESCENDING)[0]
            self.convert_text_to_speech(last_note["text"])
        except IndexError:
            self.convert_text_to_speech(JarvisPhrases.NO_NOTES)



import datetime
import os
import pyjokes
import pyttsx3
import nltk
import numpy as np
import speech_recognition as sr

from textblob import TextBlob
from nltk.corpus import stopwords
from nltk.stem.snowball import SnowballStemmer
from .phrases import BasicPhrases


class PersonalAssistant:
    """
    A class to represent voice personal assistant with basic features.

    ...

    Attributes
    ----------
    recognizer: speech_recognition.Recognizer
        object of speech_recognition.Recognizer,  which represents a collection of speech recognition functionality
    micro: speech_recognition.Microphone
        object of speech_recognition.Microphone,  which represents a physical microphone on the computer
    engine:
        text to speech engine
    stemmer: nltk.stem.snowball.SnowballStemmer
        object of SnowballStemmer used in text analyze
    working: bool
        variable, which represent if personal assistant is working
    name: str
        name of the personal assistant
    commands: dict
        dictionary where are stored all implemented features
    """
    def __init__(self, name: str):
        """
        Creates a new ``PersonalAssistant`` instance, which represents voice personal assistant with basic features.

        Parameters
        ----------
            name : str
                name of the personal assistant
        """
        self.recognizer = sr.Recognizer()
        self.micro = sr.Microphone()
        self.engine = pyttsx3.init()
        self.engine.setProperty('rate', 125)
        voices = self.engine.getProperty('voices')
        self.engine.setProperty('voice', voices[1].id)
        self.stemmer = SnowballStemmer("english")
        self.working = True
        self.name = name
        self.commands = {
            "introduce": self.introduce_yourself,
            "stop": self.stop,
            "joke": self.joke,
            "end": self.end,
        }

    def convert_text_to_speech(self, message):
        """
        String representation of parameter "message" will be converted to speech using self.engine

        :param message: str
        :return: None
        """
        self.engine.say(message)
        self.engine.runAndWait()

    def welcome(self):
        """
        Personal assistant will greet you with the appropriate greeting depending on the current time

        :return: None
        """
        hour = int(datetime.datetime.now().hour)
        greetings = {
            (0, 11): BasicPhrases.GOOD_MORNING,
            (12, 17): BasicPhrases.GOOD_AFTERNOON,
            (18, 23): BasicPhrases.GOOD_EVENING,
        }
        greeting = [value for (key, value) in greetings.items()
                    if hour in set(np.linspace(*key, dtype=int))]
        self.convert_text_to_speech(*greeting)
        self.convert_text_to_speech(BasicPhrases.HELP)

    def start(self):
        """
        Main function responsible for convert voice command to text.
        If in command there is no word equals to self.name the command is no longer process.

        :return: None
        """
        waiting_for_order = True
        self.welcome()
        with self.micro as source:
            while self.working:
                try:
                    audio = self.recognizer.listen(source, timeout=3)
                    order = self.recognizer.recognize_google(audio)
                    if self.name in order:
                        waiting_for_order = False
                        self.process_order(order, source)
                        waiting_for_order = True
                except sr.UnknownValueError:
                    if waiting_for_order is False:
                        self.convert_text_to_speech(BasicPhrases.INCOMPREHENSIBLE_SOUND)
                except sr.RequestError:
                    self.convert_text_to_speech(BasicPhrases.ERROR)
                except sr.WaitTimeoutError:
                    pass

    def introduce_yourself(self, _):
        """
        Personal assistant introduce yourself using self.name

        :return: None
        """
        self.convert_text_to_speech(f"I am {self.name}")

    def process_order(self, text: str, source: sr.Microphone):
        """
        Converted voice command is processed using nltk,TextBlob.
        Command sentence is tokenized and filtered in purpose to catch "hot" words and decide
        if sentence is connected with any implemented feature.

        :param text: str
            Voice command converted to text
        :param source: speech_recognition.Microphone
            object of speech_recognition.Microphone,  which represents a physical microphone on the computer
        :return: None
        """
        sentence_to_analyze = TextBlob(text.lower())
        polarity = sentence_to_analyze.sentiment.polarity
        if polarity < 0:
            self.convert_text_to_speech(BasicPhrases.SAD)
            self.joke(source)

        tokenized_text = sentence_to_analyze.tokenize()
        filtered_words = [word for word in tokenized_text
                          if word not in stopwords.words('english')]
        tagged = nltk.pos_tag(filtered_words)
        words = [self.stemmer.stem(word) if tag in ('VB', 'VBG') else word
                 for (word, tag) in tagged]

        for index, word in enumerate(words):
            if word in self.commands.keys():
                params = words[index + 1:]
                self.commands[word](source, params)
                break
        else:
            self.convert_text_to_speech(BasicPhrases.NO_COMMEND)

    def stop(self, _):
        """
        The personal assistant stops the program

        :return: None
        """
        self.convert_text_to_speech(BasicPhrases.GOODBYE)
        self.working = False

    def end(self, _):
        """
        The personal assistant stops the program and shuts down the computer

        :return: None
        """
        self.convert_text_to_speech(BasicPhrases.END)
        self.working = False
        os.system("shutdown /s /t 1")

    def joke(self, _):
        """
        Personal assistant will tell random joke(from pyjokes library)

        :return: None
        """
        self.convert_text_to_speech(BasicPhrases.JOKE)
        joke = pyjokes.get_joke()
        self.convert_text_to_speech(joke)



import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

VERSION = '0.1.1'
PACKAGE_NAME = 'jarvis-assistant'
AUTHOR = 'Michał Kacprzak'
AUTHOR_EMAIL = 'michal.kacprzak999@gmail.com'
URL = 'https://github.com/MichalKacprzak99/jarvis'

LICENSE = 'MIT License'
DESCRIPTION = 'Jarvis - Voice Personal Assistant'
LONG_DESCRIPTION = (HERE / "README.md").read_text()
LONG_DESC_TYPE = "text/markdown"

INSTALL_REQUIRES = [
      'playsound',
      'gTTS',
      'requests',
      'pyjokes==0.6.0',
      'pyttsx3',
      'numpy',
      'pytemperature',
      'wikipedia',
      'textblob',
      'pymongo',
      'nltk',
      'SpeechRecognition',
      'pyaudio',
      'python-dotenv',
]

setup(name=PACKAGE_NAME,
      version=VERSION,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      long_description_content_type=LONG_DESC_TYPE,
      author=AUTHOR,
      license=LICENSE,
      author_email=AUTHOR_EMAIL,
      url=URL,
      install_requires=INSTALL_REQUIRES,
      classifiers=[
            "Programming Language :: Python :: 3.6",
            "Operating System :: Microsoft :: Windows",
            "Natural Language :: English",
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 3 :: Only'
      ],
      packages=find_packages(),
      project_urls={
            'Bug Reports': 'https://github.com/MichalKacprzak99/jarvis/issues',
            'Source': 'https://github.com/MichalKacprzak99/jarvis',
      },
      )